# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.builder import Builder
from winkirip.context import Context
from winkirip.worker_chdir import WorkerChDir
from winkirip.worker_dir_struct import WorkerDirStruct
from winkirip.worker_audio import WorkerAudio
from winkirip.worker_video import WorkerVideo
from winkirip.worker_size import WorkerCalcSize
from winkirip.worker_shrink_dvd import WorkerShrinkDVD
from winkirip.worker_multiplex import WorkerMultiplex
from winkirip.worker_palette_dvd import WorkerPaletteDVD
from winkirip.worker_subtitles import WorkerSubtitles
from winkirip.worker_audio_compress import WorkerAudioCompress
from winkirip.worker_audio_video_dvd import WorkerAudioVideoDVD
from winkirip.worker_author_dvd import WorkerAuthorDVD
from winkirip.worker_create_iso import WorkerCreateISO
from winkirip.worker_link import WorkerLinkSource
from winkirip.worker_cleanup import WorkerCleanup

class BuilderDVD(Builder):
    def __init__(self, project):
        Builder.__init__(self, self.__build)

        self.__project = project
        return

    # -- private interface --

    def __build(self):
        simple = True

        # if we have to rip just one audio stream and no subtitles, we can
        # combine the audio and video steps
        containers = self.__project.get_dest()
        for container in containers:
            for title in container:
                audio_list = title.get_audio()
                subtitle_list = title.get_subtitles()
                if len(audio_list) > 1 or len(subtitle_list) > 0:
                    simple = False
                    blah = 0
                    break

                if not simple:
                    break

        # we cannot use the simplified procedure with tcrequant
        cfg = self.__project.get_config()
        shrink_method = cfg.get_video_shrink_method_dvd()
        if shrink_method == 'requantize':
            simple = False

        if not simple:
            self.__extended()
        else:
            self.__extended()
            # todo: enable? it's fine with one source container but won't work with more
            #self.__simple()

        return True

    def __extended(self):
        context = Context()
        context.set_project(self.__project)

        # preparation steps
        chdir = WorkerChDir(context)
        self.add_worker(chdir)

        dirstruct = WorkerDirStruct(context)
        self.add_worker(dirstruct)

        # aquisition steps
        for source_container in self.__project.get_source():
            source_id = source_container.get_id()

            link = WorkerLinkSource(context, source_id)
            self.add_worker(link)

            audio = WorkerAudio(context, source_id)
            self.add_worker(audio)

            acompress = WorkerAudioCompress(context, source_id)
            self.add_worker(acompress)

            palette = WorkerPaletteDVD(context, source_id)
            self.add_worker(palette)

            subtitles = WorkerSubtitles(context, source_id)
            self.add_worker(subtitles)

            video = WorkerVideo(context, source_id)
            self.add_worker(video)

            #eject = WorkerEjectMedia(context, source_id)
            #self.add_worker(eject)

        # conversion steps
        sizes = WorkerCalcSize(context)
        self.add_worker(sizes)

        shrink = WorkerShrinkDVD(context)
        self.add_worker(shrink)

        mplex = WorkerMultiplex(context)
        self.add_worker(mplex)

        author = WorkerAuthorDVD(context)
        self.add_worker(author)

        image = WorkerCreateISO(context)
        self.add_worker(image)

        # remove /tmp/source_X, divx2pass.log, audio/, video/ etc.
        cleanup = WorkerCleanup(context)
        self.add_worker(cleanup)
        return True

    def __simple(self):
        # todo: multiple sources
        context = Context()
        context.set_project(self.__project)

        chdir = WorkerChDir(context)
        self.add_worker(chdir)

        dirs = WorkerDirStruct(context)
        self.add_worker(dirs)

        link = WorkerLinkSource(context)
        self.add_worker(link)

        video = WorkerAudioVideoDVD(context)
        self.add_worker(video)

        author = WorkerAuthorDVD(context)
        self.add_worker(author)

        return True


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
