# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.builder import Builder
from winkirip.context import Context
from winkirip.worker_chdir import WorkerChDir
from winkirip.worker_dir_struct import WorkerDirStruct
from winkirip.worker_audio import WorkerAudio
from winkirip.worker_video import WorkerVideo
from winkirip.worker_chapters_tags import WorkerChaptersTags
from winkirip.worker_size import WorkerCalcSize
from winkirip.worker_multiplex import WorkerMultiplex
from winkirip.worker_subtitles import WorkerSubtitles
from winkirip.worker_audio_compress import WorkerAudioCompress
from winkirip.worker_link import WorkerLinkSource
from winkirip.worker_cleanup import WorkerCleanup

class BuilderFile(Builder):
    def __init__(self, project):
        Builder.__init__(self, self.__build)

        self.__project = project
        return

    # -- private interface --

    def __build(self):
        context = Context()
        context.set_project(self.__project)

        # preparation steps
        chdir = WorkerChDir(context)
        self.add_worker(chdir)

        dirs = WorkerDirStruct(context)
        self.add_worker(dirs)

        # create links to the source containers
        for source_container in self.__project.get_source():
            source_id = source_container.get_id()

            link = WorkerLinkSource(context, source_id)
            self.add_worker(link)

        # audio and subtitle aquisition
        for source_container in self.__project.get_source():
            source_id = source_container.get_id()

            audio = WorkerAudio(context, source_id)
            self.add_worker(audio)

            acompress = WorkerAudioCompress(context, source_id)
            self.add_worker(acompress)

            subtitles = WorkerSubtitles(context, source_id)
            self.add_worker(subtitles)

            #eject = WorkerEjectMedia(context, source_id)
            #self.add_worker(eject)

        sizes = WorkerCalcSize(context)
        self.add_worker(sizes)

        # video aquisition
        for source_container in self.__project.get_source():
            source_id = source_container.get_id()

            video = WorkerVideo(context, source_id)
            self.add_worker(video)

            #eject = WorkerEjectMedia(context, source_id)
            #self.add_worker(eject)

        chapters_tags = WorkerChaptersTags(context)
        self.add_worker(chapters_tags)

        mplex = WorkerMultiplex(context)
        self.add_worker(mplex)

        # remove /tmp/source_X, divx2pass.log, audio/, video/ etc.
        cleanup = WorkerCleanup(context)
        self.add_worker(cleanup)
        return True


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
