# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import events_pending, main_iteration
from os import kill
from commands import getoutput
from popen2 import Popen3
from winkirip.lib import debug, error, to_int

class Command:
    def __init__(self, command, callback = None, finish = None):
        self.__command = command
        self.__callback = None
        self.__finish = None
        if callback and callable(callback):
            self.__callback = callback
        if finish and callable(finish):
            self.__finish = finish
        self.__bufsize = 64
        self.__cmd_obj = None
        self.__pid = -1
        self.__output_line = ''
        self.__result = -1
        self.__pure_command = ''
        #self.__any_output = False
        return

    # -- public interface --

    def get_status(self):
        status = -1
        if self.__cmd_obj:
            status = self.__cmd_obj.poll()
        else:
            status = self.__result
        return status

    def set_bufsize(self, bufsize):
        self.__bufsize = bufsize
        return

    #def use_any_output(self, use = True):
    #   self.__any_output = use
    #   return

    def stop(self):
        self.__kill()

        self.__cmd_obj = None
        return

    def run(self):
        self.__build_pure_name()

        self.__cmd_obj = Popen3(self.__command, False, self.__bufsize)
        self.__pid = self.__cmd_obj.pid

        text = None
        while self.__cmd_obj and self.__cmd_obj.poll() == -1:
            text = self.__cmd_obj.fromchild.read(self.__bufsize)
            line = self.__build_line(text)

            #if line:
            #   print '<<-- %s -->>' % line

            if self.__callback and line:
                self.__callback(line)

            #elif self.__callback and self.__any_output:
            #   self.__callback(text)

            while events_pending():
                main_iteration(False)

        # process the remaining output
        if self.__cmd_obj:
            text = self.__cmd_obj.fromchild.read()
            pos2 = 0
            while pos2 < len(text):
                pos1 = pos2
                pos2 += self.__bufsize
                if pos2 > len(text):
                    pos2 = len(text)

                line = self.__build_line(text[pos1 : pos2])
                if self.__callback and line:
                    self.__callback(line)

        debug('Command.run()', 'stopping command')
        self.stop()
        if self.__finish:
            debug('Command.run()', 'finishing command')
            self.__finish()
        return

    def to_stdin(self, cmd):
        if not self.__cmd_obj:
            debug('Command.to_stdin()', 'in pipe not open', error)
            return

        self.__cmd_obj.tochild.write(cmd)
        self.__cmd_obj.tochild.flush()
        return

    # -- helpers --

    def __build_pure_name(self):
        if not self.__command or not len(self.__command):
            debug('Command.__build_pure_name()', 'no self.command', error)
            return

        self.__pure_command = self.__command.replace('"', '')
        return  

    def __build_line(self, text):
        result = None

        eol_pos = text.find('\n')
        eol_pos1 = text.find('\r')
        if eol_pos1 > -1 and eol_pos < 0:
            eol_pos = eol_pos1

        if eol_pos > -1:
            self.__output_line += text[ : eol_pos]
            result = self.__output_line
            self.__output_line = text[eol_pos : ]
        else:
            self.__output_line += text

        return result

    def __kill(self):
        if self.__cmd_obj:
            self.__result = self.__cmd_obj.poll()

        if self.__pid > -1:
            try:
                kill(self.__pid, 1)
                print 'killing: %d' % self.__pid
            except:
                pass

        # crude but effective...
        pid = self.__find_pid()
        while pid != -1:
            try:
                print 'more killing: %d' % pid
                kill(pid, 1)
            except:
                debug('Command.__kill()', 'error killing proccess', error)
            pid = self.__find_pid()

        if self.__cmd_obj:
            self.__cmd_obj.tochild.close()
            self.__cmd_obj.fromchild.close()
        return

    def __find_pid(self):
        output = getoutput('ps -Af')
        lines = output.split('\n')

        for line in lines:
            parts = line.split()
            fixed_line = ''.join(parts)

            pos = fixed_line.find(self.__pure_command)
            if pos < 0:
                continue

            pid = to_int(parts[1])
            if pid != -1:
                return pid

        return -1

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
