# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from container_stream import Stream, TYPE_SUBTITLE
from winkirip.lib import sec2str
from copy import deepcopy

SUBTITLE_UNKNOWN = -1
SUBTITLE_VOBSUB = 0
SUBTITLE_SRT = 1
SUBTITLE_SUB = 2
SUBTITLE_IMGFILE = 3

class Subtitle(Stream):
    def __init__(self, src = None):
        self.__language = None
        self.__length = 0
        self.__content = None
        self.__filename = None
        self.__id = -1
        self.__title_id = -1
        self.__container_id = -1
        self.__descr = None
        self.__format = SUBTITLE_UNKNOWN
        self.__source = None

        if src:
            self.__language = deepcopy(src.__language)
            self.__length = src.__length
            self.__content = deepcopy(src.__content)
            self.__filename = deepcopy(src.__filename)
            self.__id = src.__id
            self.__title_id = src.__title_id
            self.__container_id = src.__container_id
            self.__descr = deepcopy(src.__descr)
            self.__format = src.__format
        return

    # -- methods inherited from Stream --

    def stream_type(self):
        return TYPE_SUBTITLE

    # -- get/set --

    def get_language(self):
        return self.__language

    def get_length(self):
        return self.__length

    def get_content(self):
        return self.__content

    def get_filename(self):
        return self.__filename

    def get_id(self):
        return self.__id

    def get_title_id(self):
        return self.__title_id

    def get_container_id(self):
        return self.__container_id

    def get_descr(self):
        return self.__descr

    def get_format(self):
        return self.__format

    def get_source(self):
        return self.__source

    def set_language(self, language):
        self.__language = language
        return

    def set_length(self, length):
        self.__length = length
        return

    def set_content(self, content):
        self.__content = content
        return

    def set_filename(self, filename):
        self.__filename = filename
        return

    def set_id(self, id):
        self.__id = id
        return

    def set_title_id(self, title_id):
        self.__title_id = title_id
        return

    def set_container_id(self, container_id):
        self.__container_id = container_id
        return

    def set_descr(self, descr):
        self.__descr = descr
        return

    def set_format(self, format):
        self.__format = format
        return

    def set_source(self, source):
        if isinstance(source, Subtitle):
            self.__source = source
        else:
            assert(False)
        return

    def reset_source(self):
        self.__source = None
        return

    # -- public interface --

    def copy(self):
        return Subtitle(self)

    def get_formatted_length(self):
        return sec2str(self.__length)

    def __repr__(self):
        result = '    Subtitle(%s, %d)\n' % (self.__filename, self.__id)
        result += '      lang: %s, len: %d, content: %s, descr: %s, format: %d, [%d/%d]\n' % (self.__language, self.__length, self.__content, self.__descr, self.__format, self.__container_id, self.__title_id)
        return result

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
