# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from winkirip.scan import Scan
from winkirip.parser_dvd import LsDVDParser
from winkirip.parser_mplayer_dvd import MplayerDVDParser
from winkirip.command import Command
from winkirip.device_info import DeviceInfo
from winkirip.lib import set_quotes


class ScanDVD(Scan):

    def __init__(self, iso_codes, container):
        Scan.__init__(self, container)
        self.__lines = []
        self.__device_file_name = None
        self.__iso_codes = iso_codes
        return

    # -- methods inherited from scan.Scan --

    def run(self):
        if not self.__lsdvd():
            return False

        # arbitrary value at the beginning
        # (we don't know how many titles the DVD is going to have)
        self.__set_progress(1, 20)

        lsdvd_parser = LsDVDParser(self.__lines, self.__iso_codes, self.get_container())
        lsdvd_parser.run()

        if self.is_canceled():
            return True

        # lsdvd lies about some titles having more audio/subtitle languages
        # so we try to fix this information using mplayer
        self.reset()
        if not self.__mplayer():
            return False

        return True


    def reset(self):
        Scan.reset(self)
        self.__lines = []
        return

    def set_device(self, device):
        if isinstance(device, DeviceInfo):
            self.__device_file_name = device.get_full_device_file()
        else:
            self.__device_file_name = device
        return

    # -- helpers --

    def __lsdvd_fake(self):
    #def __lsdvd(self):
        f = open('/home/velko/wfx.lsdvd', 'r')
        raw = f.read()
        f.close()
        self.__lines = raw.split('\n')
        self.__lines.pop()
        for line in self.__lines:
            print line
        return True

    #def __lsdvd_orig(self):
    def __lsdvd(self):
        cmd_line = 'LC_ALL=C lsdvd -x'
        if self.__device_file_name:
            cmd_line = 'LC_ALL=C lsdvd -x %s' % self.__device_file_name
        cmd = Command(cmd_line, self.__accumulate_lines)
        cmd.set_bufsize(20)
        cmd.run()
        if cmd and cmd.get_status() != 0:
            self.reset()
            return False

        return True

    def __mplayer(self):
        cont = self.get_container()
        title_count = len(cont)
        title_index = 1
        for title in cont:
            cmd_line = ('LC_ALL=C mplayer -v -identify -frames 0 dvd://%d' % title_index)
            if self.__device_file_name:
                cmd_line += (' -dvd-device %s' % set_quotes(self.__device_file_name))
            title_index += 1

            self.__set_progress(title_index - 1, title_count)

            cmd = Command(cmd_line, self.__accumulate_lines)
            cmd.set_bufsize(5)
            cmd.run()
            if cmd and cmd.get_status() != 0:
                self.reset()
                return False

            if self.is_canceled():
                return True

            mplayer_parser = MplayerDVDParser(self.__lines, self.__iso_codes, title, cont)
            if not mplayer_parser.run():
                self.reset()
                return False

            self.reset()

        return True

    def __set_progress(self, cur, count):
        res = float(cur) / float(count)

        if res < 0.0:
            res = 0.0
        elif res > 1.0:
            res = 1.0

        self.set_progress(res)
        return

    def __accumulate_lines(self, line):
        self.__lines.append(line)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
