# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import STOCK_YES, STOCK_NO, ICON_SIZE_BUTTON
from gtk.glade import XML
from winkirip.lib import build_path, winki_path, check_widgets, debug, error
from winkirip.lib import textcombo_setup, textcombo_get_value, textcombo_set_value
from winkirip.container import CONTAINER_UNKNOWN, FILE_MATROSKA, FILE_AVI
from winkirip.container_video import VIDEO_MPEG1, VIDEO_MPEG2, VIDEO_MPEG4
from winkirip.container_video import VIDEO_XVID
from winkirip.container_audio import AUDIO_PCM, AUDIO_AC3, AUDIO_OGG
from winkirip.container_audio import AUDIO_MP2, AUDIO_MP3, AUDIO_FLAC
from winkirip.tool_checker import TOOL_EXTERNAL, TOOL_MENCODER

class SelectSettings:
    def __init__(self, config, tool_checker):
        self.__xml = None
        self.__config = config
        self.__tool_checker = tool_checker
        self.__ignore_update = False

        xml_path = build_path([winki_path, 'glade', 'settings.glade'])
        try:
            self.__xml = XML(xml_path)
        except:
            print 'SelectSettings: Cannot load "window1" from %s' % (xml_path)
            raise SystemExit

        dic = {'on_button_cancel_clicked' : self.on_cancel,
               'on_button_ok_clicked' : self.on_ok,
               'on_combobox_video_resolution_dvd_changed' : self.on_video_resolution_dvd_changed,
               'on_combobox_video_format_dvd_changed' : self.on_dvd_changed,
               'on_combobox_video_aspect_dvd_changed' : self.on_dvd_changed,
               'on_combobox_video_encoding_dvd_changed' : self.on_dvd_changed,
               'on_combobox_audio_format_dvd_changed' : self.on_dvd_changed,
               'on_combobox_dvd_presets_changed' : self.on_dvd_presets_changed,
               'on_combobox_file_format_changed' : self.on_file_format_changed,
               'on_combobox_video_format_file_changed' : self.on_file_changed,
               'on_combobox_video_resolution_file_changed' : self.on_file_changed,
               'on_combobox_video_aspect_file_changed' : self.on_file_changed,
               'on_combobox_video_encoding_file_changed' : self.on_file_changed,
               'on_combobox_audio_format_file_changed' : self.on_audio_format_file_changed,
               'on_combobox_audio_quality_file_changed' : self.on_audio_quality_file_changed,
               'on_combobox_file_presets_changed' : self.on_file_presets_changed}
        self.__xml.signal_autoconnect(dic)

        check_widgets(self.__xml, ['button_cancel', 'button_ok'])

        self.__dialog = self.__xml.get_widget('window1')
        self.__dialog.connect('delete_event', self.no_quit)

        self.__init_controls()
        return

    # -- signal handlers --

    def no_quit(self, button, event = None):
        "disallow close"
        return True

    def on_ok(self, widget, event = None):
        self.hide()
        self.__update_config()
        return True
        
    def on_cancel(self, widget, event = None):
        self.hide()
        return True

    def on_video_resolution_dvd_changed(self, widget, event = None):
        text = textcombo_get_value(widget)
        if not len(text):
            return False

        video_aspects_dvd = [('4/3', '4/3')]
        video_aspect_dvd = '4/3'
        if text == '720x576':
            video_aspects_dvd = [(_('Original'), 'original'),
                                 ('16/9', '16/9'),
                                 ('4/3', '4/3')]
            video_aspect_dvd = self.__config.get_video_aspect_dvd()

        combo_video_aspect_dvd = self.__xml.get_widget('combobox_video_aspect_dvd')
        textcombo_setup(combo_video_aspect_dvd, video_aspects_dvd, video_aspect_dvd)
        return True

    def on_dvd_changed(self, widget, event = None):
        if not self.__ignore_update:
            self.__combo_set_value('combobox_dvd_presets', ['none'])
        return True

    def on_dvd_presets_changed(self, widget, event = None):
        self.__ignore_update = True
        option = widget.get_active_text()

        if option == _('Best quality'):
            self.__combo_set_value('combobox_video_format_dvd', [VIDEO_MPEG2])
            self.__combo_set_value('combobox_video_resolution_dvd', ['720x576'])
            self.__combo_set_value('combobox_video_aspect_dvd', ['original'])
            self.__combo_set_value('combobox_video_encoding_dvd', ['2 pass'])
            # pcm have the best quality but it's just a waste in most cases...
            #self.__combo_set_value('combobox_audio_format_dvd', ['AC3', 'MP2', 'PCM'])
            self.__combo_set_value('combobox_audio_format_dvd', [AUDIO_AC3, AUDIO_MP2])
        elif option == _('Fastest encoding'):
            self.__combo_set_value('combobox_video_format_dvd', [VIDEO_MPEG2])
            self.__combo_set_value('combobox_video_resolution_dvd', ['720x576'])
            self.__combo_set_value('combobox_video_aspect_dvd', ['original'])
            self.__combo_set_value('combobox_video_encoding_dvd', ['requantize', '1 pass'])
            self.__combo_set_value('combobox_audio_format_dvd', [AUDIO_AC3, AUDIO_MP2])
        self.__ignore_update = False
        return True

    def on_file_format_changed(self, widget, event = None):
        combo_audio_format_file = self.__xml.get_widget('combobox_audio_format_file')
        combo_video_format_file = self.__xml.get_widget('combobox_video_format_file')
        file_format = textcombo_get_value(widget)

        audio_format_file = self.__config.get_audio_format_file()
        audio_formats_file = self.__create_audio_values(file_format)
        textcombo_setup(combo_audio_format_file, audio_formats_file, audio_format_file)

        video_format_file = self.__config.get_video_format_file()
        video_formats_file = self.__create_video_values(file_format)
        textcombo_setup(combo_video_format_file, video_formats_file, video_format_file)
        return True

    def on_audio_format_file_changed(self, widget, event = None):
        value = textcombo_get_value(widget)
        combo = self.__xml.get_widget('combobox_audio_quality_file')

        self.__setup_file_quality_combo(combo, value)
        return True

    def on_audio_quality_file_changed(self, widget, event = None):
        value = textcombo_get_value(widget)

        combo = self.__xml.get_widget('combobox_audio_format_file')
        aformat = textcombo_get_value(combo)

        if aformat == AUDIO_OGG:
            self.__config.set_ogg_quality(value)
        elif aformat == AUDIO_MP3:
            self.__config.set_mp3_quality(value)
        elif aformat == AUDIO_MP2:
            self.__config.set_mp2_quality(value)
        elif aformat == AUDIO_AC3:
            self.__config.set_ac3_quality(value)
        return True

    def on_file_changed(self, widget, event = None):
        if not self.__ignore_update:
            self.__combo_set_value('combobox_file_presets', ['none'])
        return True

    def on_file_presets_changed(self, widget, event = None):
        self.__ignore_update = True
        option = widget.get_active_text()

        if option == _('Best quality'):
            self.__combo_set_value('combobox_video_format_file', [VIDEO_MPEG2, VIDEO_XVID, VIDEO_MPEG4])
            self.__combo_set_value('combobox_video_resolution_file', ['original'])
            self.__combo_set_value('combobox_video_aspect_file', ['original'])
            self.__combo_set_value('combobox_video_encoding_file', ['2 pass'])
            self.__combo_set_value('combobox_audio_format_file', [AUDIO_AC3, AUDIO_OGG, AUDIO_MP2, AUDIO_MP3])
        elif option == _('Fastest encoding'):
            self.__combo_set_value('combobox_video_format_file', [VIDEO_XVID, VIDEO_MPEG4])
            self.__combo_set_value('combobox_video_resolution_file', ['original'])
            self.__combo_set_value('combobox_video_aspect_file', ['original'])
            self.__combo_set_value('combobox_video_encoding_file', ['1 pass'])
            self.__combo_set_value('combobox_audio_format_file', [AUDIO_MP3, AUDIO_OGG, AUDIO_MP2, AUDIO_AC3])
        elif option == _('Maximum compression'):
            self.__combo_set_value('combobox_video_format_file', [VIDEO_XVID, VIDEO_MPEG4, VIDEO_MPEG2])
            self.__combo_set_value('combobox_video_resolution_file', ['original'])
            self.__combo_set_value('combobox_video_aspect_file', ['original'])
            self.__combo_set_value('combobox_video_encoding_file', ['2 pass'])
            self.__combo_set_value('combobox_audio_format_file', [AUDIO_OGG, AUDIO_MP3, AUDIO_MP2, AUDIO_AC3])
        self.__ignore_update = False
        return True

    # -- public interface --

    def show(self):
        self.__dialog.show()
        return

    def hide(self):
        self.__dialog.hide()
        return

    # -- helpers --

    def __init_controls(self):
        if not self.__config:
            return

        self.__init_dvd_tab()
        self.__init_file_tab()
        self.__init_programs_tab()
        self.__init_misc()
        return

    def __init_dvd_tab(self):
        combo_video_format_dvd = self.__xml.get_widget('combobox_video_format_dvd')
        video_format_dvd = self.__config.get_video_format_dvd()
        # todo: add mpeg1
        video_formats_dvd = self.__create_values([('MPEG-2', VIDEO_MPEG2, 'lavc')])
        textcombo_setup(combo_video_format_dvd, video_formats_dvd, video_format_dvd)

        combo_video_resolution_dvd = self.__xml.get_widget('combobox_video_resolution_dvd')
        video_resolutions_dvd = [('720x576', '720x576'),
                                 ('704x576', '704x576'),
                                 ('352x576', '352x576'),
                                 ('352x288', '352x288')]
        video_resolution_dvd = self.__config.get_video_resolution_dvd()
        if video_format_dvd == VIDEO_MPEG1:
            video_resolutions_dvd = [('352x288', '352x288')]
            video_resolution_dvd  = '352x288'
        textcombo_setup(combo_video_resolution_dvd, video_resolutions_dvd, video_resolution_dvd)

        combo_video_shrink = self.__xml.get_widget('combobox_video_encoding_dvd')
        video_shrink = self.__config.get_video_shrink_method_dvd()
        desired = [(_('Requantize (fastest encoding)'), 'requantize', 'tcrequant'),
                   (_('1 pass reencode'), '1 pass', 'mencoder'),
                   (_('2 pass reencode (best quality)'), '2 pass', 'mencoder')]
        shrink_methods = self.__create_values(desired)
        textcombo_setup(combo_video_shrink, shrink_methods, video_shrink)

        combo_audio_format_dvd = self.__xml.get_widget('combobox_audio_format_dvd')
        audio_format_dvd = self.__config.get_audio_format_dvd()
        desired = [('MP2', AUDIO_MP2, 'mp2enc'),
                   ('AC3', AUDIO_AC3, 'ffmpeg')]
                   #('PCM', AUDIO_PCM, None)]
        audio_formats_dvd = self.__create_values(desired)
        textcombo_setup(combo_audio_format_dvd, audio_formats_dvd, audio_format_dvd)

        combo_dvd_presets = self.__xml.get_widget('combobox_dvd_presets')
        dvd_preset_values = [(_('None'), 'none'),
                             (_('Best quality'), 'quality'),
                             (_('Fastest encoding'), 'fast')]
        textcombo_setup(combo_dvd_presets , dvd_preset_values, 'none')
        return

    def __init_file_tab(self):
        file_format = self.__config.get_file_encoding()
        combo_video_format_file = self.__xml.get_widget('combobox_video_format_file')
        video_format_file = self.__config.get_video_format_file()
        video_formats_file = self.__create_video_values(file_format)
        textcombo_setup(combo_video_format_file, video_formats_file, video_format_file)

        combo_video_resolution_file = self.__xml.get_widget('combobox_video_resolution_file')
        video_resolutions_file = [(_('Original'), 'original'),
                                  ('720x576', '720x576'),
                                  ('704x576', '704x576'),
                                  ('352x576', '352x576'),
                                  ('352x288', '352x288'),
                                  ('384x288 (divx 4:3)', '384x288'),
                                  ('512x288 (divx 16:9)', '512x288')]
        video_resolution_file = self.__config.get_video_resolution_file()
        textcombo_setup(combo_video_resolution_file, video_resolutions_file, video_resolution_file)

        combo_video_aspect_file = self.__xml.get_widget('combobox_video_aspect_file')
        video_aspect_file = self.__config.get_video_aspect_file()
        video_aspects_file = [(_('Original'), 'original'),
                              ('16/9', '16/9'),
                              ('4/3', '4/3')]
        textcombo_setup(combo_video_aspect_file, video_aspects_file, video_aspect_file)

        combo_video_shrink = self.__xml.get_widget('combobox_video_encoding_file')
        video_shrink = self.__config.get_video_shrink_method_file()
        desired = [(_('1 pass reencode (faster)'), '1 pass', 'mencoder'),
                   (_('2 pass reencode (better quality)'), '2 pass', 'mencoder')]
        shrink_methods = self.__create_values(desired)
        textcombo_setup(combo_video_shrink, shrink_methods, video_shrink)

        combo_file_format = self.__xml.get_widget('combobox_file_format')
        desired = [(_('Matroska (superior)'), FILE_MATROSKA, 'mkvmerge'),
                   (_('AVI (inferior)'), FILE_AVI, 'avimerge')]
        file_formats = self.__create_values(desired)
        textcombo_setup(combo_file_format, file_formats, file_format)

        combo_audio_format_file = self.__xml.get_widget('combobox_audio_format_file')
        audio_format_file = self.__config.get_audio_format_file()
        audio_formats_file = self.__create_audio_values(file_format)
        textcombo_setup(combo_audio_format_file, audio_formats_file, audio_format_file)

        combo_audio_quality_file = self.__xml.get_widget('combobox_audio_quality_file')
        combo_audio_format = self.__xml.get_widget('combobox_audio_format_file')
        value = textcombo_get_value(combo_audio_format)
        self.__setup_file_quality_combo(combo_audio_quality_file, value)

        combo_file_presets = self.__xml.get_widget('combobox_file_presets')
        file_preset_values = [(_('None'), 'none'),
                              (_('Best quality'), 'quality'),
                              (_('Fastest encoding'), 'fast'),
                              (_('Maximum compression'), 'compress')]
        textcombo_setup(combo_file_presets , file_preset_values, 'none')
        return

    def __init_programs_tab(self):
        button = self.__xml.get_widget('button_ok')
        yes_pixbuf = button.render_icon(STOCK_YES, ICON_SIZE_BUTTON)
        no_pixbuf = button.render_icon(STOCK_NO, ICON_SIZE_BUTTON)

        self.__init_program_entry('mplayer', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('mencoder', yes_pixbuf, no_pixbuf)
        self.__init_program('divx', 'lavc', yes_pixbuf, no_pixbuf)
        self.__init_program('xvid', 'xvid', yes_pixbuf, no_pixbuf)
        self.__init_program('mpeg2', 'lavc', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('lsdvd', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('oggenc', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('ffmpeg', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('mkvmerge', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('avimerge', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('mplex', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('dvdauthor', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('tcrequant', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('tcextract', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('title', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('lindex', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('paldump', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('lame', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('mp2enc', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('spuunmux', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('spumux', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('mkisofs', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('cdrecord', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('hal-device', yes_pixbuf, no_pixbuf)
        return

    def __init_misc(self):
        checkbutton_ch_rip = self.__xml.get_widget('checkbutton_chapter_rip')
        enabled = self.__config.get_chapter_rip()
        checkbutton_ch_rip.set_active(enabled)
        return

    def __init_program_entry(self, tool_name, yes_pixbuf, no_pixbuf):
        entry = self.__xml.get_widget('entry_%s' % tool_name)
        image = self.__xml.get_widget('image_%s' % tool_name)

        lc_all_prefix = False
        location = ''
        pixbuf = no_pixbuf
        if self.__tool_checker.has_tool(tool_name):
            location = self.__tool_checker.get_tool_location(tool_name, lc_all_prefix)
            pixbuf = yes_pixbuf

        entry.set_text(location)
        image.set_from_pixbuf(pixbuf)
        return

    def __init_program(self, ui_name, tool_name, yes_pixbuf, no_pixbuf):
        image = self.__xml.get_widget('image_%s' % ui_name)

        lc_all_prefix = False
        pixbuf = no_pixbuf
        if self.__tool_checker.has_tool(tool_name, TOOL_MENCODER):
            location = self.__tool_checker.get_tool_location(tool_name, lc_all_prefix)
            pixbuf = yes_pixbuf

        image.set_from_pixbuf(pixbuf)
        return

    def __update_config(self):
        if not self.__config:
            assert(False)
            return

        self.__update_dvd_tab()
        self.__update_file_tab()
        self.__update_programs_tab()
        self.__update_misc()

        if not self.__config.write():
            assert(False)
        return

    def __update_dvd_tab(self):
        combo = self.__xml.get_widget('combobox_video_format_dvd')
        value = textcombo_get_value(combo)
        self.__config.set_video_format_dvd(value)

        combo = self.__xml.get_widget('combobox_video_resolution_dvd')
        value = textcombo_get_value(combo)
        self.__config.set_video_resolution_dvd(value)

        combo = self.__xml.get_widget('combobox_video_aspect_dvd')
        value = textcombo_get_value(combo)
        self.__config.set_video_aspect_dvd(value)

        combo = self.__xml.get_widget('combobox_video_encoding_dvd')
        value = textcombo_get_value(combo)
        self.__config.set_video_shrink_method_dvd(value)

        combo = self.__xml.get_widget('combobox_audio_format_dvd')
        value = textcombo_get_value(combo)
        self.__config.set_audio_format_dvd(value)
        return

    def __update_file_tab(self):
        combo = self.__xml.get_widget('combobox_file_format')
        value = textcombo_get_value(combo)
        self.__config.set_file_encoding(value)

        combo = self.__xml.get_widget('combobox_video_format_file')
        value = textcombo_get_value(combo)
        self.__config.set_video_format_file(value)

        combo = self.__xml.get_widget('combobox_video_resolution_file')
        value = textcombo_get_value(combo)
        self.__config.set_video_resolution_file(value)

        combo = self.__xml.get_widget('combobox_video_aspect_file')
        value = textcombo_get_value(combo)
        self.__config.set_video_aspect_file(value)

        combo = self.__xml.get_widget('combobox_video_encoding_file')
        value = textcombo_get_value(combo)
        self.__config.set_video_shrink_method_file(value)

        combo = self.__xml.get_widget('combobox_audio_quality_file')
        quality = textcombo_get_value(combo)

        combo = self.__xml.get_widget('combobox_audio_format_file')
        value = textcombo_get_value(combo)
        self.__config.set_audio_format_file(value)

        if value == AUDIO_OGG:
            self.__config.set_ogg_quality(quality)
        elif value == AUDIO_MP3:
            self.__config.set_mp3_quality(quality)
        elif value == AUDIO_MP2:
            self.__config.set_mp2_quality(quality)
        elif value == AUDIO_AC3:
            self.__config.set_ac3_quality(quality)
        else:
            debug('SelectSettings.__update_file_tab()', 'unexpected file format %s' % value, error)
        return

    def __update_programs_tab(self):
        entry = self.__xml.get_widget('entry_mplayer')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('mplayer', location)

        entry = self.__xml.get_widget('entry_mencoder')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('mencoder', location)

        entry = self.__xml.get_widget('entry_lsdvd')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('lsdvd', location)

        entry = self.__xml.get_widget('entry_oggenc')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('oggenc', location)

        entry = self.__xml.get_widget('entry_ffmpeg')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('ffmpeg', location)

        entry = self.__xml.get_widget('entry_mkvmerge')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('mkvmerge', location)

        entry = self.__xml.get_widget('entry_avimerge')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('avimerge', location)

        entry = self.__xml.get_widget('entry_mplex')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('mplex', location)

        entry = self.__xml.get_widget('entry_dvdauthor')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('dvdauthor', location)

        entry = self.__xml.get_widget('entry_tcrequant')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('tcrequant', location)

        entry = self.__xml.get_widget('entry_tcextract')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('tcextract', location)

        entry = self.__xml.get_widget('entry_title')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('title', location)

        entry = self.__xml.get_widget('entry_lindex')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('lindex', location)

        entry = self.__xml.get_widget('entry_paldump')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('paldump', location)

        entry = self.__xml.get_widget('entry_lame')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('lame', location)

        entry = self.__xml.get_widget('entry_mp2enc')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('mp2enc', location)

        entry = self.__xml.get_widget('entry_spuunmux')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('spuunmux', location)

        entry = self.__xml.get_widget('entry_spumux')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('spumux', location)

        entry = self.__xml.get_widget('entry_mkisofs')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('mkisofs', location)

        entry = self.__xml.get_widget('entry_cdrecord')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('cdrecord', location)

        entry = self.__xml.get_widget('entry_hal-device')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('hal-device', location)
        return

    def __update_misc(self):
        checkbutton_ch_rip = self.__xml.get_widget('checkbutton_chapter_rip')
        enabled = checkbutton_ch_rip.get_active()
        self.__config.set_chapter_rip(enabled)
        return


    def __combo_set_value(self, combo_name, keys):
        combo = self.__xml.get_widget(combo_name)
        if not textcombo_set_value(combo, keys):
            return False

        return True

    def __create_video_values(self, file_format):
        result = []

        desired = [(_('DivX (best results with low bitrates)'),VIDEO_MPEG4, 'lavc'),
                   (_('XviD (best results with low bitrates)'), VIDEO_XVID, 'xvid')]

        if file_format == FILE_MATROSKA:
            desired.append(tuple((_('MPEG-2 (best results with high bitrates)'), VIDEO_MPEG2, 'lavc')))

        return self.__create_values(desired)

    def __create_audio_values(self, file_format):
        result = []

        desired = [('MP3', AUDIO_MP3, 'lame'),
                   ('AC3', AUDIO_AC3, 'ffmpeg')]
                   #('PCM', AUDIO_PCM, None)]

        if file_format == FILE_MATROSKA:
            desired.append(tuple(('OGG', AUDIO_OGG, 'oggenc')))
            desired.append(tuple(('MP2', AUDIO_MP2, 'mp2enc')))

        return self.__create_values(desired)

    def __create_values(self, tool_value_list):
        result = []
        for item in tool_value_list:
            value, key, tool = item

            ttype = TOOL_EXTERNAL
            if tool in ['xvid', 'lavc', 'toolame', 'mp3lame', 'twolame']:
                ttype = TOOL_MENCODER

            if tool:
                if self.__tool_checker.has_tool(tool, ttype):
                    result.append(tuple((value, key)))
            else:
                result.append(tuple((value, key)))

        return result

    def __setup_file_quality_combo(self, combo, value):
        combo.set_sensitive(True)

        if value == AUDIO_OGG:
            ogg_quality = self.__config.get_ogg_quality()
            audio_qualities_file = [(_('64 kb/s'), '64'),
                                    (_('96 kb/s'), '96'),
                                    (_('128 kb/s default'), '128'),
                                    (_('160 kb/s'), '160'),
                                    (_('192 kb/s'), '192'),
                                    (_('224 kb/s'), '224'),
                                    (_('256 kb/s'), '256'),
                                    (_('320 kb/s'), '320')]
            textcombo_setup(combo, audio_qualities_file, ogg_quality)
        elif value == AUDIO_MP3:
            mp3_quality = self.__config.get_mp3_quality()
            audio_qualities_file = [(_('64 kb/s'), '64'),
                                    (_('96 kb/s'), '96'),
                                    (_('128 kb/s default'), '128'),
                                    (_('160 kb/s'), '160'),
                                    (_('192 kb/s'), '192'),
                                    (_('224 kb/s'), '224'),
                                    (_('256 kb/s'), '256'),
                                    (_('320 kb/s'), '320')]
            textcombo_setup(combo, audio_qualities_file, mp3_quality)
        elif value == AUDIO_MP2:
            mp2_quality = self.__config.get_mp2_quality()
            audio_qualities_file = [(_('64 kb/s'), '64'),
                                    (_('96 kb/s'), '96'),
                                    (_('128 kb/s'), '128'),
                                    (_('160 kb/s'), '160'),
                                    (_('192 kb/s'), '192'),
                                    (_('224 kb/s default'), '224'),
                                    (_('256 kb/s'), '256'),
                                    (_('320 kb/s'), '320'),
                                    (_('384 kb/s'), '384'),
                                    (_('512 kb/s'), '512')]
            textcombo_setup(combo, audio_qualities_file, mp2_quality)
        elif value == AUDIO_AC3:
            ac3_quality = self.__config.get_ac3_quality()
            audio_qualities_file = [(_('64 kb/s'), '64'),
                                    (_('96 kb/s'), '96'),
                                    (_('112 kb/s'), '112'),
                                    (_('128 kb/s'), '128'),
                                    (_('224 kb/s default'), '224'),
                                    (_('384 kb/s'), '384'),
                                    (_('512 kb/s'), '512')]
            textcombo_setup(combo, audio_qualities_file, ac3_quality)
        else:
            combo.set_sensitive(False)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
