# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand
from winkirip.lib import sec2str, fix_name, fix_fname
from winkirip.writer_xml import WriterXML
from winkirip.constructor_mpegvideo import ConstructorMpegVideo
from winkirip.container import CONTAINER_FILE
from os.path import join

class WorkerAuthorDVD(Worker):
    def __init__(self, context):
        Worker.__init__(self, context, self.__create, _('Author DVD'))
        self.__create_log_text = ''
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create(self, command_list):
        self.__create_log_text = 'create dvdauthor xml files\n'
        ui_text = (_('Creating DVD descriptions'))
        ui_command = UICommand(self.__exec_create_xml, ui_text)
        ui_command.set_create_log_callback(self.__create_log)
        ui_command.set_exec_log_callback(self.__exec_log)
        command_list.append(ui_command)

        dest = self.get_context().get_dest()
        for disc_nr in range(len(dest)):
            #cont = self.get_context().get_dest()[disc_nr]

            fname = 'disc.xml'
            if len(dest) > 1:
                fname = 'disc%d.xml' % (disc_nr + 1)

            # todo
            # dvdauthor does not like streams created by tcextract (this is
            # the case we haven't requantified the extracted stream and haven
            # mixed it with subtitles or audio) or created by tcextract and
            # requantifyed by tcrequant. The funny thing is - mplex fixes such
            # streams. It is very unlikely that the user does not want a dvd
            # without audio, so I'll postpone this problem yet...

            #ui_text = (_('Filtering video (disk %d :: title %d)') % (disc_nr + 1, title_nr + 1))
            #ui_command = UICommand(self.__exec_filter_video, ui_text)
            #ui_command.set_create_log_callback(self.__create_log)
            #ui_command.set_exec_log_callback(self.__exec_log)
            #ui_command.set_data([disc_nr, title_nr])
            #command_list.append(ui_command)

            ui_text = (_('Authoring disk %d') % (disc_nr + 1))
            ui_command = UICommand(self.__exec_author, ui_text)
            ui_command.set_create_log_callback(self.__create_log)
            ui_command.set_exec_log_callback(self.__exec_log)
            ui_command.set_data([fname])
            command_list.append(ui_command)

        return True

    def __exec_author(self, command_data, command):
        assert(len(command_data) == 1)
        fname, = command_data

        tool_checker = self.get_context().get_tool_checker()
        if not tool_checker.has_tool('dvdauthor'):
            self.__exec_log_text = ('dvdauthor not found\n')
            command.set_error(_('No %s installation found') % 'dvdauthor')
            return False

        program = tool_checker.get_tool_location('dvdauthor')
        cmd_line = '%s -x %s' % (program, fname)
        self.__exec_log_text = '%s\n' % cmd_line

        return command.run_command(cmd_line)

    def __exec_create_xml(self, command_data, command):
        self.__exec_log_text = ''

        dest = self.get_context().get_dest()
        for disc_nr in range(len(dest)):
            cont = self.get_context().get_dest()[disc_nr]

            fname = 'disc.xml'
            out_dir = 'DISC'
            if cont.get_name():
                out_dir = fix_fname(cont.get_name()).upper()

            if len(dest) > 1:
                out_dir += ('%d' % (disc_nr + 1))
                fname = 'disc%d.xml' % (disc_nr + 1)

            writer = WriterXML(fname)
            self.__start_xml(writer, out_dir)
            self.__start_menu_xml(writer)
            self.__end_menu_xml(writer)
            self.__start_titleset_xml(writer)

            writer.increment()

            result = True
            for title_nr in range(len(cont)):
                if title_nr == 0:
                    result &= self.__video_options_xml(writer, cont[title_nr])
                    # a bug? in dvdauthor forces us to use only one title's audio & subs
                    # todo: pick the title with most audio/subtitle streams
                    result &= self.__audio_options_longest_list_xml(writer, cont)
                    result &= self.__subtitle_options_longest_list_xml(writer, cont)
                #result &= self.__audio_options_xml(writer, cont[title_nr])
                #result &= self.__subtitle_options_xml(writer, cont[title_nr])
                result &= self.__title_xml(writer, cont[title_nr])

            self.__end_titleset_xml(writer)
            self.__end_xml(writer)

            writer.write()
            self.__exec_log_text += ('created %s\n' % fname)

        return result

    def __start_xml(self, writer, output_dir):
        writer.stag('dvdauthor', {'dest' : output_dir})
        return

    def __end_xml(self, writer):
        writer.decrement()
        writer.etag('dvdauthor')
        return

    def __start_menu_xml(self, writer):
        # no menus right now
        writer.increment()
        writer.tag('vmgm', {})
        return

    def __end_menu_xml(self, writer):
        # no menus right now
        return

    def __start_titleset_xml(self, writer):
        writer.stag('titleset', {})

        writer.increment()
        writer.stag('titles', {})
        return

    def __end_titleset_xml(self, writer):
        writer.decrement()
        writer.etag('titles')

        writer.decrement()
        writer.etag('titleset')
        return

    def __video_options_xml(self, writer, title):
        vstreams = title.get_video()
        if len(vstreams) < 1:
            return False

        # todo: video[0]
        vstream = vstreams[0]

        vformat = vstream.get_format_str()
        vaspect = vstream.get_aspect('dvdauthor')

        cfg = self.get_context().get_config()
        sx, sy = cfg.get_video_resolution_dvd().split('x')
        vres = ('%sx%s' % (sx, sy))
        wscr = 'nopanscan'

        opts = {'widescreen' : wscr, 'resolution' : vres, 'aspect' : vaspect, 'format' : vformat}
        writer.tag('video', opts)

        return True

    def __audio_options_xml(self, writer, title):
        streams = title.get_audio()
        for audio in streams:
            alang = audio.get_language()
            writer.tag('audio', {'lang' : alang})

        return True

    def __subtitle_options_xml(self, writer, title):
        streams = title.get_subtitles()
        for subtitle in streams:
            slang = subtitle.get_language()
            writer.tag('subpicture', {'lang' : slang})

        return True

    def __audio_options_longest_list_xml(self, writer, cont):
        streams = []
        for title in cont:
            if len(title.get_audio()) > len(streams):
                streams = title.get_audio()

        for audio in streams:
            alang = audio.get_language()
            writer.tag('audio', {'lang' : alang})

        return True

    def __subtitle_options_longest_list_xml(self, writer, cont):
        streams = []
        for title in cont:
            if len(title.get_subtitles()) > len(streams):
                streams = title.get_subtitles()

        for subtitle in streams:
            slang = subtitle.get_language()
            writer.tag('subpicture', {'lang' : slang})

        return True

    def __title_xml(self, writer, title):
        palette_file = title.get_palette_file()
        vstreams = title.get_video()
        if len(vstreams) < 1:
            return False

        writer.stag('pgc', {'palette' : palette_file})
        writer.increment()

        ch_start = title.get_first_chapter_id()
        ch_end = title.get_last_chapter_id()
        chapt_info = self.expand_chapter_list(ch_start, ch_end, vstreams[0].get_filename())

        for start_ch, end_ch, vname in chapt_info:
            chapters = self.__create_chapter_times(title, start_ch, end_ch)
            writer.tag('vob', {'chapters' : chapters, 'file' : vname})

        writer.decrement()
        writer.etag('pgc')

        return True

    def __create_chapter_times(self, title, start_ch, end_ch):
        total = 0.
        time_str = sec2str(total)

        chapters = title.get_chapters()
        if len(chapters) < 1:
            return None

        # chapter rip
        if start_ch == end_ch:
            if start_ch > 1:
                for i in range(start_ch - 1):
                    total += chapters[i].get_length()
                time_str = sec2str(total)

            time_str += (',%s' % sec2str(total + chapters[start_ch - 1].get_length()))
            return time_str

        # title rip
        for chapter in chapters:
            total += chapter.get_length()
            time_str += (',%s' % sec2str(total))

        return time_str

    def __exec_filter_video(self, command_data, command):
        assert(len(command_data) == 2)
        disc_nr, title_nr = command_data

        cont = self.get_context().get_dest()[disc_nr]
        title = cont[title_nr]

        fname = 'video/disc%d_title%d_filtered.m2v' % (disc_nr + 1, title_nr + 1)
        tool_checker = self.get_context().get_project().get_tool_checker()

        constr = ConstructorMpegVideo(tool_checker)
        constr.set_source(title.get_video_at(0).get_filename())
        constr.set_source_type(CONTAINER_FILE)
        constr.set_outname(fname)

        success, shell_cmd = constr.construct()
        if not success:
            command.set_error(_('You may get help in the winki forum located at http://www.winki-the-ripper.de'))
            return False

        self.__exec_log_text = ('%s\n' % shell_cmd)

        result = command.run_command(shell_cmd)
        if result:
            title.get_video_at(0).set_filename(fname)

        return result

    def __parser_callback(self, ui_command, text):
        progress = 0.0
        estimation = None
        return progress, estimation

    def __create_log(self, fobj):
        if not self.__create_log_text or not len(self.__create_log_text):
            fobj.write('no authoring information yet\n')
            return

        fobj.write('%s' % self.__create_log_text)
        return

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            fobj.write('no authoring information yet\n')
            return

        fobj.write('%s' % self.__exec_log_text)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
