# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.lib import mkdir
from winkirip.command_ui import UICommand
from os import chdir

class WorkerChDir(Worker):
    def __init__(self, context):
        Worker.__init__(self, context, self.__create, _('Change working folder'))
        self.__create_log_text = ''
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create(self, command_list):
        dest = self.get_context().get_project().get_dest_dir()
        if not dest:
            return False

        dest = self.get_context().get_project().get_dest_dir()
        ui_text = (_('Changing working folder to %s') % dest)
        self.__create_log_text = ('chdir %s\n' % dest)

        ui_command = UICommand(self.__execute, ui_text)
        ui_command.set_create_log_callback(self.__create_log)
        ui_command.set_exec_log_callback(self.__exec_log)
        command_list.append(ui_command)
        return True

    def __execute(self, command_data, command):
        dest = self.get_context().get_project().get_dest_dir()

        if not mkdir(dest):
            self.__exec_log_text = ('failed to create %s\n' % dest)
            command.set_error(_('Unable to create folder %s') % dest)
            return False

        chdir('%s' % dest)
        self.__exec_log_text = ('chdir %s\n' % dest)
        return True

    def __create_log(self, fobj):
        if not self.__create_log_text or not len(self.__create_log_text):
            fobj.write('no dest dir information yet\n')
            return

        fobj.write('%s' % self.__create_log_text)
        return

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            fobj.write('no dest dir information yet\n')
            return

        fobj.write('%s' % self.__exec_log_text)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
