# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand
from winkirip.lib import fix_fname
from winkirip.container import CONTAINER_DVD
from os.path import join, split

class WorkerCreateISO(Worker):
    def __init__(self, context):
        Worker.__init__(self, context, self.__create, _('Create ready to burn ISO images'))
        self.__create_log_text = ''
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create(self, command_list):
        self.__create_log_text = 'create ISO images for burning\n'

        dest = self.get_context().get_dest()
        for disc_nr in range(len(dest)):
            ui_text = (_('Creating ISO image %d') % (disc_nr + 1))
            ui_command = UICommand(self.__exec_iso, ui_text)
            ui_command.set_create_log_callback(self.__create_log)
            ui_command.set_exec_log_callback(self.__exec_log)
            ui_command.set_data([disc_nr])
            command_list.append(ui_command)

        return True

    def __exec_iso(self, command_data, command):
        assert(len(command_data) == 1)
        disc_nr, = command_data

        tool_checker = self.get_context().get_tool_checker()
        if not tool_checker.has_tool('mkisofs'):
            self.__exec_log_text = ('mkisofs not found\n')
            command.set_error(_('No %s installation found') % 'mkisofs')
            return False

        cont = self.get_context().get_dest()[disc_nr]
        if len(cont) < 1:
            self.__exec_log_text = ('no titles in container\n')
            command.set_error(_('Cannot create ISO image for this media'))
            return False
        fname = cont[0].get_video_at(0).get_filename()

        iso_name = 'disk_image'
        mkisofs_opts = '-J -r'

        if cont.get_type() == CONTAINER_DVD:
            mkisofs_opts = '-dvd-video'
            fname = 'DISC'
            if cont.get_name():
                fname = fix_fname(cont.get_name()).upper()
            if len(self.get_context().get_dest()) > 1:
                fname += ('%d' % (disc_nr + 1))

        if cont.get_name():
            mkisofs_opts += (' -V "%s"' % cont.get_name())

        if len(self.get_context().get_dest()) > 1:
            iso_name += ('_%d' % disc_nr)
        iso_name += '.iso'

        program = tool_checker.get_tool_location('mkisofs')
        cmd_line = '%s -o %s %s %s' % (program, iso_name, mkisofs_opts, fname)
        self.__exec_log_text = '%s\n' % cmd_line

        return command.run_command(cmd_line)

    def __parser_callback(self, ui_command, text):
        progress = 0.0
        estimation = None
        return progress, estimation

    def __create_log(self, fobj):
        if not self.__create_log_text or not len(self.__create_log_text):
            fobj.write('no image creation information yet\n')
            return

        fobj.write('%s' % self.__create_log_text)
        return

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            fobj.write('no image creation information yet\n')
            return

        fobj.write('%s' % self.__exec_log_text)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
