# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand
from winkirip.lib import does_path_exist, debug
from os import symlink, unlink

class WorkerLinkSource(Worker):
    def __init__(self, context, source_id):
        Worker.__init__(self, context, self.__create, _('Link source'))
        self.__dev_link = ('/tmp/source_%d' % source_id)
        self.__source_id = source_id
        self.__create_log_text = ''
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create(self, command_list):
        device = None
        for source_container in self.get_context().get_project().get_source():
            if source_container.get_id() == self.__source_id:
                device = source_container.get_device()
                source_container.set_device(self.__dev_link)
                break

        if not device:
            return False

        self.__create_log_text = ('ln -sf "%s" %s\n' % (device, self.__dev_link))
        ui_command = UICommand(self.__execute, _('Linking source file to %s') % self.__dev_link)
        ui_command.set_create_log_callback(self.__create_log)
        ui_command.set_exec_log_callback(self.__exec_log)
        ui_command.set_data([device])
        command_list.append(ui_command)
        return True

    def __execute(self, command_data, command):
        assert len(command_data) == 1
        device, = command_data

        if does_path_exist(self.__dev_link):
            debug('WorkerLinkSource.__execute()', '%s already exist: removing it' % self.__dev_link)
            self.__exec_log_text = ('File %s already exist: removing it\n' % self.__dev_link)
            unlink(self.__dev_link)
            #command.set_error(_('File %s already exist') % self.__dev_link)
            #return False

        self.__exec_log_text = ('ln -sf "%s" %s\n' % (device, self.__dev_link))
        symlink(device, self.__dev_link)

        return True

    def __create_log(self, fobj):
        if not self.__create_log_text or not len(self.__create_log_text):
            fobj.write('no link information yet\n')
            return

        fobj.write('%s' % self.__create_log_text)
        return

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            fobj.write('no link information yet\n')
            return

        fobj.write('%s' % self.__exec_log_text)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
