# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand
from winkirip.container import CONTAINER_DVD, CONTAINER_FILE, FILE_AVI, FILE_MATROSKA
from winkirip.lib import debug, error, move, fix_fname, build_path, mkdir
from os.path import split

class WorkerMultiplex(Worker):
    def __init__(self, context):
        dest = context.get_project().get_dest()
        create_func = None

        if dest[0].get_type() == CONTAINER_DVD:
            create_func = self.__create_dvd
        elif dest[0].get_type() == CONTAINER_FILE:
            create_func = self.__create_file

        Worker.__init__(self, context, create_func, _('Multiplex streams'))
        self.__create_log_text = ''
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create_dvd(self, command_list):
        self.__create_log_text = ''

        iso_codes = self.get_context().get_project().get_iso_codes()

        for disc_nr in range(len(self.get_context().get_dest())):
            cont = self.get_context().get_dest()[disc_nr]

            title_nr = 0
            for title in cont:
                outname = 'video/disc%d_title%d.m2v' % (disc_nr + 1, title_nr + 1)
                self.__create_log_text += ('mplex: %s\n' % outname)

                astreams = title.get_audio()
                sstreams = title.get_subtitles()

                if len(astreams) < 1 and len(sstreams) < 1:
                    continue

                if len(astreams) > 0:
                    ui_text = (_('Multiplexing video and all audio streams (disk %d :: title %d)') % (disc_nr + 1, title_nr + 1))
                    ui_command = UICommand(self.__exec_dvd_video, ui_text, True)
                    ui_command.set_create_log_callback(self.__create_log)
                    ui_command.set_exec_log_callback(self.__exec_log)
                    ui_command.set_data([disc_nr, title_nr, outname])
                    command_list.append(ui_command)

                subpos = 0
                for subtitle in sstreams:
                    ch_start = title.get_first_chapter_id()
                    ch_end = title.get_last_chapter_id()
                    # todo: video[0]
                    chapt_info = self.expand_chapter_list(ch_start, ch_end, outname)

                    for start_ch, end_ch, vname in chapt_info:
                        tname = self.get_name_for_chapters(start_ch, end_ch, 'video/temp.vob')

                        ui_text = _('Creating temporary video file')
                        ui_command = UICommand(self.__rename_video, ui_text)
                        ui_command.set_create_log_callback(self.__create_log)
                        ui_command.set_exec_log_callback(self.__exec_log)
                        ui_command.set_data([vname, tname])
                        command_list.append(ui_command)

                        sname = _('Unknown')
                        lang = iso_codes.find_iso_639_1(subtitle.get_language())
                        if lang:
                            sname = lang.get_name()

                        ui_text = (_('Multiplexing video and %s subtitles (disk %d :: title %d)') % (sname, disc_nr + 1, title_nr + 1))
                        ui_command = UICommand(self.__mplex_subtitle, ui_text)
                        ui_command.set_create_log_callback(self.__create_log)
                        ui_command.set_exec_log_callback(self.__exec_log)
                        ui_command.set_data([disc_nr, title_nr, subtitle.get_id(), tname, start_ch, end_ch])
                        command_list.append(ui_command)

                        ui_text = _('Removing temporary video file')
                        ui_command = UICommand(self.__unlink_temp, ui_text)
                        ui_command.set_create_log_callback(self.__create_log)
                        ui_command.set_exec_log_callback(self.__exec_log)
                        ui_command.set_data([tname])
                        command_list.append(ui_command)

                    subpos += 1
                title_nr += 1

        return True

    def __exec_dvd_video(self, command_data, command):
        assert(len(command_data) == 3)
        disc_nr, title_nr, outname = command_data

        cont = self.get_context().get_dest()[disc_nr]
        title = cont[title_nr]
        ch_start = title.get_first_chapter_id()
        ch_end = title.get_last_chapter_id()
        chapt_info = self.expand_chapter_list(ch_start, ch_end, outname)

        result = True
        for start_ch, end_ch, fname in chapt_info:
            shell_cmd = 'LC_ALL=C mplex -f 8 -o %s ' % fname

            # todo: video[0]
            vfile = self.get_name_for_chapters(start_ch, end_ch, title.get_video_at(0).get_filename())
            shell_cmd += vfile

            for astream in title.get_audio():
                afile = self.get_name_for_chapters(start_ch, end_ch, astream.get_filename())
                shell_cmd += ' '
                shell_cmd += afile

            self.__exec_log_text = ('%s\n' % shell_cmd)

            if not command.run_command(shell_cmd):
                result = False
                break

        if result:
            title.get_video_at(0).set_filename(outname)
        else:
            command.set_error(_('You may get help in the winki forum located at http://www.winki-the-ripper.de'))

        return result

    def __rename_video(self, command_data, command):
        assert(len(command_data) == 2)
        vname, tname = command_data

        shell_cmd = 'mv %s %s' % (vname, tname)
        self.__exec_log_text += ('%s\n' % shell_cmd)

        return move(vname, tname)

    def __mplex_subtitle(self, command_data, command):
        template = 'LC_ALL=C spumux -m dvd -s %d %s < %s > %s'

        assert(len(command_data) == 6)
        disc_nr, title_nr, sid, tname, start_ch, end_ch = command_data

        cont = self.get_context().get_dest()[disc_nr]
        title = cont[title_nr]

        vname = self.get_name_for_chapters(start_ch, end_ch, title.get_video_at(0).get_filename())

        subtitles = title.get_subtitles()
        subpos = title.get_subtitle_pos(sid)
        # the subtitle was removed from the container because it does not contain any data
        # and this would confuse mplex
        if subpos < 0:
            self.__exec_log_text = ('skipping subtitle %d - it has been removed\n' % sid)
            move(tname, vname)
            return True

        subtitle = subtitles[subpos]
        sname = self.get_name_for_chapters(start_ch, end_ch, subtitle.get_filename())

        shell_cmd = template % (subpos, sname, tname, vname)
        self.__exec_log_text = ('%s\n' % shell_cmd)

        return command.run_command(shell_cmd)


    def __unlink_temp(self, command_data, command):
        assert(len(command_data) == 1)
        tname, = command_data

        self.__exec_log_text += ('rm %s\n' % tname)

        self.unlink_temp([tname])
        return True

    def __create_file_name(self, disc_nr, ext, title_nr = -1):
        name = self.get_context().get_project().get_dest()[disc_nr].get_name()
        if not name or not len(name):
            name = 'disk'

        title = ''
        if title_nr > -1:
            title = '_%d' % (title_nr + 1)

        output_fname = ''
        if len(self.get_context().get_dest()) == 1:
            output_fname = '%s%s.%s' % (fix_fname(name), title, ext)
        else:
            output_fname = '%s_%d%s.%s' % (fix_fname(name), disc_nr + 1, title, ext)

        return build_path(['%d' % (disc_nr + 1), output_fname])

    def __create_file(self, command_list):
        config = self.get_context().get_project().get_config()
        file_encoding = config.get_file_encoding()

        if file_encoding == FILE_AVI:
            for disc_nr in range(len(self.get_context().get_dest())):
                cont = self.get_context().get_dest()[disc_nr]
                for title_nr in range(len(cont)):
                    output_fname = self.__create_file_name(disc_nr, 'avi', title_nr)
                    ui_command = UICommand(self.__exec_avi_file, _('Multiplexing %s') % output_fname)
                    ui_command.set_create_log_callback(self.__create_log)
                    ui_command.set_exec_log_callback(self.__exec_log)
                    ui_command.set_data([disc_nr, title_nr, output_fname])
                    command_list.append(ui_command)

        elif file_encoding == FILE_MATROSKA:
            for disc_nr in range(len(self.get_context().get_dest())):
                cont = self.get_context().get_dest()[disc_nr]
                for title_nr in range(len(cont)):
                    output_fname = self.__create_file_name(disc_nr, 'mkv', title_nr)
                    ui_command = UICommand(self.__exec_mkv_file, _('Multiplexing %s') % output_fname)
                    ui_command.set_create_log_callback(self.__create_log)
                    ui_command.set_exec_log_callback(self.__exec_log)
                    ui_command.set_data([disc_nr, title_nr, output_fname])
                    command_list.append(ui_command)
        else:
            assert(False)
            print 'unexpected file encoding requested'
            return False

        return True

    def __exec_avi_file(self, command_data, command):
        assert(len(command_data) == 3)
        disc_nr, title_nr, avi_fname = command_data

        cont = self.get_context().get_dest()[disc_nr]
        title = cont[title_nr]
        tool_checker = self.get_context().get_project().get_tool_checker()

        prog_name = ''
        if tool_checker.has_tool('avimerge'):
            prog_name = tool_checker.get_tool_location('avimerge')
        else:
            command.set_error(_('No %s installation found') % 'avimerge')
            return False

        out_dir = build_path(['%d' % (disc_nr + 1)])
        if not mkdir(out_dir):
            command.set_error(_('Cannot create output folder %s') % out_dir)
            self.__exec_log_text = ('Failed to create output directory %s\n' % out_dir)
            return False

        shell_cmd = '%s -o %s' % (prog_name, avi_fname)

        # video stream
        shell_cmd += ' -i %s' % (title.get_video_at(0).get_filename())

        # audio streams
        aid = 0
        for astream in title.get_audio():
            # avimerge supports up to 2 audio streams

            # this restriction is not implemented in the UI because the user
            # may switch the output file format any time after selecting the
            # audio languages
            if aid > 1:
                break

            afile = astream.get_filename()
            if aid == 0:
                shell_cmd += (' -p %s -A %d' % (afile, aid))
            else:
                shell_cmd += (' %s -A %d' % (afile, aid))
            aid += 1

        self.__exec_log_text = ('%s\n' % shell_cmd)

        if not command.run_command(shell_cmd):
            command.set_error(_('You may get help in the winki forum located at http://www.winki-the-ripper.de'))
            return False

        return True

    def __exec_mkv_file(self, command_data, command):
        assert(len(command_data) == 3)
        disc_nr, title_nr, mkv_fname = command_data

        cont = self.get_context().get_dest()[disc_nr]
        title = cont[title_nr]
        lang_conv = self.get_context().get_project().get_iso_codes()
        tool_checker = self.get_context().get_project().get_tool_checker()

        prog_name = ''
        if tool_checker.has_tool('mkvmerge'):
            prog_name = tool_checker.get_tool_location('mkvmerge')
        else:
            command.set_error(_('No %s installation found') % 'mkvmerge')
            return False

        out_dir = build_path(['%d' % (disc_nr + 1)])
        if not mkdir(out_dir):
            command.set_error(_('Cannot create output folder %s') % out_dir)
            self.__exec_log_text = ('Failed to create output directory %s\n' % out_dir)
            return False

        shell_cmd = '%s -o %s' % (prog_name, mkv_fname)

        file_id = 0
        aspect = title.get_video_at(0).get_float_aspect()
        if aspect < 0:
            debug('WorkerMultiplexDVD.__exec_mkv_file()', 'aspect not set', error)
            return False

        # aspect ratio for this title
        shell_cmd += ' --aspect-ratio %d:%f' % (title_nr, aspect)

        # video stream
        shell_cmd += ' -S -A %s' % (title.get_video_at(0).get_filename())
        title.get_video_at(0).set_id(file_id)
        file_id += 1

        # audio streams
        apos = 0
        for astream in title.get_audio():
            lang = lang_conv.find_iso_639_1(astream.get_language())
            alang = lang.get_iso_639_2()
            afile = astream.get_filename()
            astream.set_id(file_id)
            file_id += 1

            shell_cmd += ' --language %d:%s -D -S %s' % (title_nr, alang, afile)
            apos += 1

        # subtitle streams
        spos = 0
        for sstream in title.get_subtitles():
            lang = lang_conv.find_iso_639_1(sstream.get_language())
            slang = lang.get_iso_639_2()
            sfile = sstream.get_filename()
            sstream.set_id(file_id)
            file_id += 1

            shell_cmd += ' --language 0:%s -D -A %s' % (slang, sfile)
            spos += 1

        title_nr += 1

        tags = cont.get_tags()
        if len(tags) > 0:
            tags_fname = 'disc%d_tags.xml' % (disc_nr + 1)
            shell_cmd += ' --global-tags %s' % tags_fname

        chap_fname = 'disc%d_chapters.xml' % (disc_nr + 1)
        shell_cmd += ' --chapters %s' % chap_fname

        shell_cmd += ' --title "%s"' % cont.get_name()

        self.__exec_log_text = ('%s\n' % shell_cmd)

        success_codes = [0, 1, 256]
        result = command.run_command(shell_cmd, success_codes)

        if result:
            items = split(mkv_fname)
            if len(items) > 1:
                title.get_video_at(0).set_filename(items[0])
        else:
            command.set_error(_('You may get help in the winki forum located at http://www.winki-the-ripper.de'))

        return result

    def __get_common_stream_count(self, disc_nr):
        astream_count = -1
        sstream_count = -1
        cont = self.get_context().get_dest()[disc_nr]

        for title in cont:
            acount = len(title.get_audio())
            if astream_count < 0:
                astream_count = acount
            else:
                if astream_count != acount:
                    astream_count = min(astream_count, acount)

            scount = len(title.get_subtitles())
            if sstream_count < 0:
                sstream_count = scount
            else:
                if sstream_count != scount:
                    sstream_count = min(sstream_count, scount)

        return astream_count, sstream_count

    def match_stream_count(self, disc_nr):
        vcount = -1
        acount = -1
        scount = -1
        cont = self.get_context().get_dest()[disc_nr]
        for title in cont:
            if vcount < 0:
                vcount = len(title.get_video())
            else:
                if vcount != len(title.get_video()):
                    return False

            if acount < 0:
                acount = len(title.get_audio())
            else:
                if acount != len(title.get_audio()):
                    return False

            if scount < 0:
                scount = len(title.get_subtitles())
            else:
                if scount != len(title.get_subtitles()):
                    return False
        return True

    def __create_log(self, fobj):
        if not self.__create_log_text or not len(self.__create_log_text):
            fobj.write('no multiplex information yet\n')
            return

        fobj.write('%s' % self.__create_log_text)
        return

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            fobj.write('no multiplex information yet\n')
            return

        fobj.write('%s' % self.__exec_log_text)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
