# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand
from winkirip.container import CONTAINER_DVD, CONTAINER_FILE
from winkirip.container_video import VIDEO_MPEG2, VIDEO_MPEG4, VIDEO_XVID
from winkirip.constructor_ffmpeg4 import ConstructorFFmpeg4
from winkirip.constructor_xvid import ConstructorXvid
from winkirip.constructor_mpeg2 import ConstructorMpeg2DVD
from winkirip.constructor_tcextract import ConstructorTcextract
from winkirip.lib import debug, error

class WorkerVideo(Worker):
    def __init__(self, context, source_id):
        self.__source_container_id = source_id

        Worker.__init__(self, context, self.__create, _('Extract video'))
        self.__create_log_text = ''
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create(self, command_list):
        vlist = self.get_video_list(self.__source_container_id)

        for video in vlist:
            disc_id = video.get_container_id()
            title_id = video.get_title_id()
            ui_templ = _('Extracting video stream (disk %d :: title %d)')
            ui_text = ui_templ % (disc_id + 1, title_id + 1)

            pass_count = -1
            pass_nr = -1
            shrink_method = self.get_shrink_method(video)
            # there is a separate step for dvd video shrinking
            if self.get_dest_container_type(disc_id) == CONTAINER_DVD:
                shrink_method = 'requantize'

            if shrink_method == '1 pass':
                pass_nr = 1
                pass_count = 1

            elif shrink_method == '2 pass':
                ui_templ = _('Creating video index (disk %d :: title %d)')
                ui_text = ui_templ % (disc_id + 1, title_id + 1)
                pass_nr = 1
                pass_count = 2

            ui_command = UICommand(self.__exec, ui_text, True)
            ui_command.set_create_log_callback(self.__create_log)
            ui_command.set_exec_log_callback(self.__exec_log)
            #ui_command.set_parse_callback(self.__parse_output)
            ui_command.set_data([video, pass_nr, pass_count])
            command_list.append(ui_command)

            if shrink_method == '2 pass':
                ui_templ = _('Extracting video stream (disk %d :: title %d)')
                ui_text = ui_templ % (disc_id + 1, title_id + 1)
                pass_nr = 2

                ui_command = UICommand(self.__exec, ui_text, True)
                ui_command.set_create_log_callback(self.__create_log)
                ui_command.set_exec_log_callback(self.__exec_log)
                #ui_command.set_parse_callback(self.__parse_output)
                ui_command.set_data([video, pass_nr, pass_count])
                command_list.append(ui_command)

        return True

    def __exec(self, command_data, command):
        assert(len(command_data) == 3)
        video, pass_nr, pass_count = command_data

        source_video = video.get_source()
        cont_id = source_video.get_container_id()
        title_id = source_video.get_title_id()
        source_container = self.get_source_container(cont_id)
        source_title = self.get_source_title(cont_id, title_id)
        device = source_container.get_device()
        source_type = source_container.get_type()
        source_title_id = source_title.get_id()

        cont_id = video.get_container_id()
        title_id = video.get_title_id()
        container = self.get_dest_container(cont_id)
        title = self.get_dest_title(cont_id, title_id)
        ch_start = title.get_first_chapter_id()
        ch_end = title.get_last_chapter_id()
        outname = 'video/disc%d_title%d_video.mpeg' % (cont_id + 1, title_id + 1)

        result, vformat = self.__extract_video(command, ch_start, ch_end, container, title, outname, command_data, source_type, device, source_title_id)

        if result:
            self.__update_video(video, vformat, outname)
        else:
            command.set_error(_('You may get help in the winki forum located at http://www.winki-the-ripper.de'))

        return result

    def __extract_video(self, command, ch_start, ch_end, container, title, outname, command_data, source_type, device, source_title_id):
        video, pass_nr, pass_count = command_data
        tool_checker = self.get_context().get_tool_checker()
        cfg = self.get_context().get_config()

        vformat = VIDEO_MPEG2
        if container.get_type() == CONTAINER_DVD:
            vformat = cfg.get_video_format_dvd()
        elif container.get_type() == CONTAINER_FILE:
            vformat = cfg.get_video_format_file()
        else:
            # todo
            return False, None

        chapt_info = self.expand_chapter_list(ch_start, ch_end, outname)
        for start_ch, end_ch, fname_ch in chapt_info:
            constr = None
            if pass_count < 0:
                constr = ConstructorTcextract(tool_checker)
            else:
                if vformat == VIDEO_MPEG4:
                    constr = ConstructorFFmpeg4(tool_checker)
                elif vformat == VIDEO_XVID:
                    constr = ConstructorXvid(tool_checker)
                elif vformat == VIDEO_MPEG2:
                    constr = ConstructorMpeg2DVD(tool_checker)
                else:
                    return False, None

            constr.set_source_type(source_type)
            if source_type == CONTAINER_DVD:
                constr.set_source(device, source_title_id)
                constr.set_chapters(start_ch, end_ch)
            else:
                return False, None

            if pass_count > 0:
                constr.set_pass_count(pass_count)
                constr.set_pass_nr(pass_nr)

                aspect = '4/3'
                resol = '720x576'
                if container.get_type() == CONTAINER_DVD:
                    aspect = cfg.get_video_aspect_dvd()
                    resol = cfg.get_video_resolution_dvd()
                elif container.get_type() == CONTAINER_FILE:
                    aspect = cfg.get_video_aspect_file()
                    resol = cfg.get_video_resolution_file()

                crop = title.get_crop()
                aspect, scale_expand = self.compute_scale_expand(video, crop, aspect, resol)
                constr.set_scale_expand(scale_expand)
                constr.set_aspect(aspect)
                constr.set_vbitrate(video.get_bitrate())

            constr.set_outname(fname_ch)

            success, shell_cmd = constr.construct()
            if not success:
                debug('WorkerVideo.__extract_video()', 'command construction failed 1', error)
                command.set_error(_('Command construction failed'))
                return False, None

            self.__exec_log_text += '%s\n' % shell_cmd

            if not command.run_command(shell_cmd):
                return False, None

        return True, vformat

    def __update_video(self, video, vtype, fname, aspect = None):
        video.set_filename(fname)
        video.set_type(vtype)
        video.set_id(1) # todo

        if aspect:
            video.set_aspect(aspect)
        return

    def __create_log(self, fobj):
        if not self.__create_log_text or not len(self.__create_log_text):
            fobj.write('no video information yet\n')
            return

        fobj.write('%s' % self.__create_log_text)
        return

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            fobj.write('no video information yet\n')
            return

        fobj.write('%s' % self.__exec_log_text)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
